using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Input.Touch;
using Microsoft.Xna.Framework.Media;
using Ckknight.StateMachine;

namespace WindowsPhoneGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";

            // Frame rate is 30 fps by default for Windows Phone.
            TargetElapsedTime = TimeSpan.FromTicks(333333);

            _stateMachine = StateMachineBuilder.Create<State, Trigger>()
                .Configure(State.Alpha)
                    .Permit(Trigger.TouchPress, State.Bravo)
                    .Ignore(Trigger.TouchRelease)
                    .Permit(Trigger.BackPressed, State.Exiting)
                    .OnEntry(() => _currentColor = Color.CornflowerBlue)
                    .End()
                .Configure(State.Bravo)
                    .Permit(Trigger.TouchRelease, State.Alpha)
                    .Ignore(Trigger.TouchPress)
                    .Permit(Trigger.BackPressed, State.Exiting)
                    .OnEntry(() => _currentColor = Color.IndianRed)
                    .End()
                .Configure(State.Exiting)
                    .OnEntry(() => this.Exit())
                    .End()
                .CreateFactory()
                .Generate(State.Alpha);
        }

        enum State { Alpha, Bravo, Exiting }
        enum Trigger { TouchPress, TouchRelease, BackPressed }
        private readonly StateMachine<State, Trigger, object> _stateMachine;

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here

            base.Initialize();
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
            {
                _stateMachine.Fire(Trigger.BackPressed);
            }

#if WINDOWS_PHONE
            foreach (var touch in TouchPanel.GetState())
            {
                switch (touch.State)
                {
                    case TouchLocationState.Pressed:
                        _stateMachine.Fire(Trigger.TouchPress);
                        break;
                    case TouchLocationState.Released:
                        _stateMachine.Fire(Trigger.TouchRelease);
                        break;
                }
            }
#elif WINDOWS
            var mouseState = Mouse.GetState();
            switch (mouseState.LeftButton)
            {
                case ButtonState.Pressed:
                    _stateMachine.Fire(Trigger.TouchPress);
                    break;
                case ButtonState.Released:
                    _stateMachine.Fire(Trigger.TouchRelease);
                    break;
            }
#elif XBOX
            switch (GamePad.GetState(PlayerIndex.One).Buttons.A)
            {
                case ButtonState.Pressed:
                    _stateMachine.Fire(Trigger.TouchPress);
                    break;
                case ButtonState.Released:
                    _stateMachine.Fire(Trigger.TouchRelease);
                    break;
            }
#endif

            base.Update(gameTime);
        }

        private Color _currentColor = Color.CornflowerBlue;
        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(_currentColor);

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
